"""Test `nth` last of type selectors."""
from .. import util


class TestNthLastOfType(util.TestCase):
    """Test `nth` last of type selectors."""

    def test_nth_last_of_type(self):
        """Test `nth` last of type."""

        markup = """
        <p id="0"></p>
        <p id="1"></p>
        <span id="2"></span>
        <span id="3"></span>
        <span id="4"></span>
        <span id="5"></span>
        <span id="6"></span>
        <p id="7"></p>
        <p id="8"></p>
        <p id="9"></p>
        <p id="10"></p>
        <span id="11"></span>
        """

        self.assert_selector(
            markup,
            "p:nth-last-of-type(3)",
            ['8'],
            flags=util.HTML
        )

    def test_nth_last_of_type_complex(self):
        """Test `nth` last of type complex."""

        markup = """
        <p id="0"></p>
        <p id="1"></p>
        <span id="2"></span>
        <span id="3"></span>
        <span id="4"></span>
        <span id="5"></span>
        <span id="6"></span>
        <p id="7"></p>
        <p id="8"></p>
        <p id="9"></p>
        <p id="10"></p>
        <span id="11"></span>
        """

        self.assert_selector(
            markup,
            "p:nth-last-of-type(2n + 1)",
            ['1', '8', '10'],
            flags=util.HTML
        )



"""Test focus selector."""
from .. import util


class TestFocus(util.TestCase):
    """Test focus selector."""

    MARKUP = """
    <form action="#">
      <fieldset id='a' disabled>
        <legend>
          Simple fieldset <input type="radio" id="1" checked>
          <fieldset id='b' disabled>
            <legend>Simple fieldset <input type="radio" id="2" checked></legend>
            <input type="radio" id="3" checked>
            <label for="radio">radio</label>
          </fieldset>
        </legend>
        <fieldset id='c' disabled>
          <legend>Simple fieldset <input type="radio" id="4" checked></legend>
          <input type="radio" id="5" checked>
          <label for="radio">radio</label>
        </fieldset>
        <input type="radio" id="6" checked>
        <label for="radio">radio</label>
      </fieldset>
      <optgroup id="opt-enable">
        <option id="7" disabled>option</option>
      </optgroup>
      <optgroup id="8" disabled>
        <option id="9">option</option>
      </optgroup>
      <a href="" id="link">text</a>
    </form>
    """

    def test_focus(self):
        """Test focus."""

        self.assert_selector(
            self.MARKUP,
            "input:focus",
            [],
            flags=util.HTML
        )

    def test_not_focus(self):
        """Test not focus."""

        self.assert_selector(
            self.MARKUP,
            "input:not(:focus)",
            ["1", "2", "3", "4", "5", "6"],
            flags=util.HTML
        )



"""Test quirky behaviors."""
from . import util
from bs4 import BeautifulSoup


class TestQuirks(util.TestCase):
    """Test quirky behaviors."""

    def test_quirky_user_attrs(self):
        """Test cases where a user creates weird attributes: nested sequences."""

        html = """
        <div id="test">test</div>
        """

        soup = BeautifulSoup(html, 'html.parser')
        soup.div.attrs['user'] = [['a']]
        print(soup.div.attrs)
        self.assertTrue(soup.select_one('div[user="[\'a\']"]') is not None)



"""Test `nth` child selectors."""
from .. import util
import soupsieve as sv
from soupsieve import SelectorSyntaxError


class TestNthChild(util.TestCase):
    """Test `nth` child selectors."""

    def test_nth_child(self):
        """Test `nth` child."""

        markup = """
        <body>
        <p id="0"></p>
        <p id="1"></p>
        <span id="2"></span>
        <span id="3"></span>
        <span id="4"></span>
        <span id="5"></span>
        <span id="6"></span>
        <p id="7"></p>
        <p id="8"></p>
        <p id="9"></p>
        <p id="10"></p>
        <span id="11"></span>
        </body>
        """

        self.assert_selector(
            markup,
            "p:nth-child(-2)",
            [],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:nth-child(2)",
            ['1'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:NTH-CHILD(2)",
            ['1'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            r"p:NT\H-CH\ILD(2)",
            ['1'],
            flags=util.HTML
        )

    def test_nth_child_odd(self):
        """Test `nth` child odd."""

        markup = """
        <body>
        <p id="0"></p>
        <p id="1"></p>
        <span id="2"></span>
        <span id="3"></span>
        <span id="4"></span>
        <span id="5"></span>
        <span id="6"></span>
        <p id="7"></p>
        <p id="8"></p>
        <p id="9"></p>
        <p id="10"></p>
        <span id="11"></span>
        </body>
        """

        self.assert_selector(
            markup,
            "p:nth-child(odd)",
            ['0', '8', '10'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:nth-child(ODD)",
            ['0', '8', '10'],
            flags=util.HTML
        )

    def test_nth_child_even(self):
        """Test `nth` child even."""

        markup = """
        <body>
        <p id="0"></p>
        <p id="1"></p>
        <span id="2"></span>
        <span id="3"></span>
        <span id="4"></span>
        <span id="5"></span>
        <span id="6"></span>
        <p id="7"></p>
        <p id="8"></p>
        <p id="9"></p>
        <p id="10"></p>
        <span id="11"></span>
        </body>
        """

        self.assert_selector(
            markup,
            "p:nth-child(even)",
            ['1', '7', '9'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:nth-child(EVEN)",
            ['1', '7', '9'],
            flags=util.HTML
        )

    def test_nth_child_complex(self):
        """Test `nth` child complex."""

        markup = """
        <body>
        <p id="0"></p>
        <p id="1"></p>
        <span id="2"></span>
        <span id="3"></span>
        <span id="4"></span>
        <span id="5"></span>
        <span id="6"></span>
        <p id="7"></p>
        <p id="8"></p>
        <p id="9"></p>
        <p id="10"></p>
        <span id="11"></span>
        </body>
        """

        self.assert_selector(
            markup,
            "p:nth-child(2n-5)",
            ['0', '8', '10'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:nth-child(2N-5)",
            ['0', '8', '10'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:nth-child(-2n+20)",
            ['1', '7', '9'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:nth-child(50n-20)",
            [],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:nth-child(-2n-2)",
            [],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:nth-child(9n - 1)",
            ['7'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:nth-child(2n + 1)",
            ['0', '8', '10'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "p:nth-child(-n+3)",
            ['0', '1'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "span:nth-child(-n+3)",
            ['2'],
            flags=util.HTML
        )

        self.assert_selector(
            markup,
            "body *:nth-child(-n+3)",
            ['0', '1', '2'],
            flags=util.HTML
        )

    def test_nth_child_no_parent(self):
        """Test `nth` child with no parent."""

        markup = """
        <body>
        <p id="0"></p>
        <p id="1"></p>
        <span id="2"></span>
        <span id="3"></span>
        <span id="4"></span>
        <span id="5"></span>
        <span id="6"></span>
        <p id="7"></p>
        <p id="8"></p>
        <p id="9"></p>
        <p id="10"></p>
        <span id="11"></span>
        </body>
        """

        for parser in util.available_parsers('html.parser', 'lxml', 'html5lib'):
            # Paragraph is the root. There is no document.
            markup = """<p id="1">text</p>"""
            soup = self.soup(markup, parser)
            fragment = soup.p.extract()
            self.assertTrue(sv.match("p:nth-child(1)", fragment, flags=sv.DEBUG))

    def test_nth_child_with_bad_parameters(self):
        """Test that pseudo class fails with bad parameters (basically it doesn't match)."""

        self.assert_raises(':nth-child(a)', SelectorSyntaxError)



"""Test not selectors."""
from .. import util


class TestNot(util.TestCase):
    """Test not selectors."""

    def test_multi_nested_not(self):
        """Test nested not and multiple selectors."""

        markup = """
        <div>
        <p id="0">Some text <span id="1"> in a paragraph</span>.</p>
        <a id="2" href="http://google.com">Link</a>
        <span id="3">Direct child</span>
        <pre id="pre">
        <span id="4">Child 1</span>
        <span id="5">Child 2</span>
        <span id="6">Child 3</span>
        </pre>
        </div>
        """

        self.assert_selector(
            markup,
            'div :not(p, :not([id=\\35]))',
            ['5'],
            flags=util.HTML
        )



"""Test only of type selectors."""
from .. import util


class TestOnlyOfType(util.TestCase):
    """Test only of type selectors."""

    def test_only_of_type(self):
        """Test only of type."""

        markup = """
        <div id="0">
            <p id="1"></p>
        </div>
        <span id="2"></span>
        <span id="3"></span>
        <p id="4"></p>
        <span id="5"></span>
        <span id="6"></span>
        <div>
            <p id="7"></p>
            <p id="8"></p>
        </div>
        """

        self.assert_selector(
            markup,
            "p:only-of-type",
            ['1', '4'],
            flags=util.HTML
        )
